@@ -284,3 +284,42 @@ const struct ppm_name_value splice_flags[] = {
 	{"SPLICE_F_GIFT", PPM_SPLICE_F_GIFT},
 	{ },
 };
+
+const struct ppm_name_value quotactl_dqi_flags[] = {
+	{"DQF_NONE", PPM_DQF_NONE},
+	{"V1_DQF_RSQUASH", PPM_V1_DQF_RSQUASH},
+	{ }
+};
+
+const struct ppm_name_value quotactl_cmds[] = {
+	{"Q_QUOTAON", PPM_Q_QUOTAON},
+	{"Q_QUOTAOFF", PPM_Q_QUOTAOFF},
+	{"Q_GETFMT", PPM_Q_GETFMT},
+	{"Q_GETINFO", PPM_Q_GETINFO},
+	{"Q_SETINFO", PPM_Q_SETINFO},
+	{"Q_GETQUOTA", PPM_Q_GETQUOTA},
+	{"Q_SETQUOTA", PPM_Q_SETQUOTA},
+	{"Q_SYNC", PPM_Q_SYNC},
+	{"Q_XQUOTAON", PPM_Q_XQUOTAON},
+	{"Q_XQUOTAOFF", PPM_Q_XQUOTAOFF},
+	{"Q_XGETQUOTA", PPM_Q_XGETQUOTA},
+	{"Q_XSETQLIM", PPM_Q_XSETQLIM},
+	{"Q_XGETQSTAT", PPM_Q_XGETQSTAT},
+	{"Q_XQUOTARM", PPM_Q_XQUOTARM},
+	{"Q_XQUOTASYNC", PPM_Q_XQUOTASYNC},
+	{ },
+};
+
+const struct ppm_name_value quotactl_types[] = {
+	{"USRQUOTA", PPM_USRQUOTA},
+	{"GRPQUOTA", PPM_GRPQUOTA},
+	{ },
+};
+
+const struct ppm_name_value quotactl_quota_fmts[] = {
+	{"QFMT_NOT_USED", PPM_QFMT_NOT_USED},
+	{"QFMT_VFS_OLD", PPM_QFMT_VFS_OLD},
+	{"QFMT_VFS_V0", PPM_QFMT_VFS_V0},
+	{"QFMT_VFS_V1", PPM_QFMT_VFS_V1},
+	{ }
+};
