@@ -34,6 +34,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <linux/vmalloc.h>
 #include <linux/wait.h>
 #include <linux/tracepoint.h>
+#include <linux/jiffies.h>
 #include <asm/syscall.h>
 #include <net/sock.h>
 
@@ -45,7 +46,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "ppm.h"
 
 MODULE_LICENSE("GPL");
-MODULE_AUTHOR("Draios");
+MODULE_AUTHOR("sysdig inc");
 
 #define PPM_DEVICE_NAME "sysdig"
 #define PPE_DEVICE_NAME PPM_DEVICE_NAME "-events"
@@ -66,6 +67,28 @@ struct ppm_device {
 	wait_queue_head_t read_queue;
 };
 
+struct event_data_t {
+	enum ppm_capture_category category;
+
+	union {
+		struct {
+			struct pt_regs *regs;
+			long id;
+		} syscall_data;
+
+		struct {
+			struct task_struct *sched_prev;
+			struct task_struct *sched_next;
+		} context_data;
+
+		struct {
+			int sig;
+			struct siginfo *info;
+			struct k_sigaction *ka;
+		} signal_data;
+	} event_info;
+};
+
 /*
  * FORWARD DECLARATIONS
  */
@@ -75,21 +98,16 @@ static long ppm_ioctl(struct file *f, unsigned int cmd, unsigned long arg);
 static int ppm_mmap(struct file *filp, struct vm_area_struct *vma);
 static int record_event_consumer(struct ppm_consumer_t *consumer,
 	enum ppm_event_type event_type,
-	struct pt_regs *regs,
-	long id,
 	enum syscall_flags drop_flags,
-	struct task_struct *sched_prev,
-	struct task_struct *sched_next,
-	struct timespec *ts);
+	struct timespec *ts,
+	struct event_data_t *event_datap);
 static void record_event_all_consumers(enum ppm_event_type event_type,
-	struct pt_regs *regs,
-	long id,
 	enum syscall_flags drop_flags,
-	struct task_struct *sched_prev,
-	struct task_struct *sched_next);
+	struct event_data_t *event_datap);
 static int init_ring_buffer(struct ppm_ring_buffer_context *ring);
 static void free_ring_buffer(struct ppm_ring_buffer_context *ring);
 static ssize_t ppe_write(struct file *filp, const char __user *buf, size_t count, loff_t *f_pos);
+void ppm_task_cputime_adjusted(struct task_struct *p, cputime_t *ut, cputime_t *st);
 
 #ifndef CONFIG_HAVE_SYSCALL_TRACEPOINTS
  #error The kernel must have HAVE_SYSCALL_TRACEPOINTS in order for sysdig to be useful
@@ -106,6 +124,10 @@ TRACEPOINT_PROBE(sched_switch_probe, struct task_struct *prev, struct task_struc
 #endif /* (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,35)) */
 #endif /* CAPTURE_CONTEXT_SWITCHES */
 
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+TRACEPOINT_PROBE(signal_deliver_probe, int sig, struct siginfo *info, struct k_sigaction *ka);
+#endif
+
 DECLARE_BITMAP(g_events_mask, PPM_EVENT_MAX);
 static struct ppm_device *g_ppm_devs;
 static struct class *g_ppm_class;
@@ -141,6 +163,9 @@ static struct tracepoint *tp_sched_process_exit;
 #ifdef CAPTURE_CONTEXT_SWITCHES
 static struct tracepoint *tp_sched_switch;
 #endif
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+static struct tracepoint *tp_signal_deliver;
+#endif
 
 #ifdef _DEBUG
 static bool verbose = 1;
@@ -377,6 +402,14 @@ static int ppm_open(struct inode *inode, struct file *filp)
 			goto err_sched_switch;
 		}
 #endif
+
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+		ret = compat_register_trace(signal_deliver_probe, "signal_deliver", tp_signal_deliver);
+		if (ret) {
+			pr_err("can't create the signal_deliver tracepoint\n");
+			goto err_signal_deliver;
+		}
+#endif
 		g_tracepoint_registered = true;
 	}
 
@@ -384,6 +417,10 @@ static int ppm_open(struct inode *inode, struct file *filp)
 
 	goto cleanup_open;
 
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+err_signal_deliver:
+	compat_unregister_trace(sched_switch_probe, "signal_switch", tp_signal_deliver);
+#endif
 err_sched_switch:
 	compat_unregister_trace(syscall_procexit_probe, "sched_process_exit", tp_sched_process_exit);
 err_sched_procexit:
@@ -453,6 +490,9 @@ static int ppm_release(struct inode *inode, struct file *filp)
 
 #ifdef CAPTURE_CONTEXT_SWITCHES
 			compat_unregister_trace(sched_switch_probe, "sched_switch", tp_sched_switch);
+#endif
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+			compat_unregister_trace(signal_deliver_probe, "signal_deliver", tp_signal_deliver);
 #endif
 			tracepoint_synchronize_unregister();
 			g_tracepoint_registered = false;
@@ -511,6 +551,7 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	}
 	case PPM_IOCTL_DISABLE_DROPPING_MODE:
 	{
+		struct event_data_t event_data;
 		struct timespec ts;
 
 		vpr_info("PPM_IOCTL_DISABLE_DROPPING_MODE, consumer %p\n", consumer_id);
@@ -524,7 +565,11 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 		 * mode has been disabled
 		 */
 		getnstimeofday(&ts);
-		record_event_consumer(consumer, PPME_SYSDIGEVENT_E, NULL, -1, UF_NEVER_DROP, (void *)DEI_DISABLE_DROPPING, (void *)0, &ts);
+		event_data.category = PPMC_CONTEXT_SWITCH;
+		event_data.event_info.context_data.sched_prev = (void *)DEI_DISABLE_DROPPING;
+		event_data.event_info.context_data.sched_next = (void *)0;
+
+		record_event_consumer(consumer, PPME_SYSDIGEVENT_E, UF_NEVER_DROP, &ts, &event_data);
 
 		ret = 0;
 		goto cleanup_ioctl;
@@ -681,6 +726,96 @@ static long ppm_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
 	case PPM_IOCTL_GET_CURRENT_PID:
 		ret = task_tgid_nr(current);
 		goto cleanup_ioctl;
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+	case PPM_IOCTL_DISABLE_SIGNAL_DELIVER:
+	{
+		vpr_info("PPM_IOCTL_DISABLE_SIGNAL_DELIVER\n");
+		if (g_tracepoint_registered) 
+			compat_unregister_trace(signal_deliver_probe, "signal_deliver", tp_signal_deliver);
+		ret = 0;
+		goto cleanup_ioctl;
+	}
+	case PPM_IOCTL_ENABLE_SIGNAL_DELIVER:
+	{
+		vpr_info("PPM_IOCTL_ENABLE_SIGNAL_DELIVER\n");
+		if (g_tracepoint_registered)
+			compat_register_trace(signal_deliver_probe, "signal_deliver", tp_signal_deliver);
+		ret = 0;
+		goto cleanup_ioctl;
+	}
+#endif
+	case PPM_IOCTL_GET_PROCLIST:
+	{
+		struct task_struct *p, *t;
+		u64 nentries = 0;
+		struct ppm_proclist_info* pli;
+		struct ppm_proc_info pi;
+
+		if (copy_from_user(&pli, (void*)arg, sizeof(pli))) {
+			ret = -EINVAL;
+			goto cleanup_ioctl;
+		}
+
+		vpr_info("PPM_IOCTL_GET_PROCLIST, size=%d\n", (int)pli->max_entries);
+
+		rcu_read_lock();
+		
+#ifdef for_each_process_thread
+		for_each_process_thread(p, t) {
+#else
+		for_each_process(p) {
+			t = p;
+			do {
+				task_lock(p);
+#endif
+				if (nentries < pli->max_entries) {
+					cputime_t utime, stime;
+
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(3, 9, 0))
+					utime = t->utime;
+					stime = t->stime;
+#else
+					ppm_task_cputime_adjusted(t, &utime, &stime);
+#endif
+					pi.pid = t->pid;
+					pi.utime = cputime_to_clock_t(utime);
+					pi.stime = cputime_to_clock_t(stime);
+
+					if (copy_to_user((void*)&pli->entries[nentries], &pi, sizeof(struct ppm_proc_info))) {
+						rcu_read_unlock();
+						ret = -EINVAL;
+						goto cleanup_ioctl;
+					}
+				}
+
+				nentries++;
+#ifdef for_each_process_thread
+		}
+#else
+				task_unlock(p);
+			} while_each_thread(p, t);
+		}
+#endif
+
+		rcu_read_unlock();
+
+		if (copy_to_user((void*)&pli->n_entries, &nentries, sizeof(pli->n_entries))) {
+			ret = -EINVAL;
+			goto cleanup_ioctl;
+		}
+
+		if (nentries >= pli->max_entries) {
+			vpr_info("PPM_IOCTL_GET_PROCLIST: not enough space (%d avail, %d required)\n", 
+				(int)pli->max_entries,
+				(int)nentries);
+
+			ret = -ENOSPC;
+			goto cleanup_ioctl;
+		}
+
+		ret = 0;
+		goto cleanup_ioctl;
+	}
 	default:
 		ret = -ENOTTY;
 		goto cleanup_ioctl;
@@ -940,7 +1075,8 @@ static enum ppm_event_type parse_socketcall(struct event_filler_arguments *fille
 
 static inline void record_drop_e(struct ppm_consumer_t *consumer, struct timespec *ts)
 {
-	if (record_event_consumer(consumer, PPME_DROP_E, NULL, -1, UF_NEVER_DROP, NULL, NULL, ts) == 0) {
+	struct event_data_t event_data = {0};
+	if (record_event_consumer(consumer, PPME_DROP_E, UF_NEVER_DROP, ts, &event_data) == 0) {
 		consumer->need_to_insert_drop_e = 1;
 	} else {
 		if (consumer->need_to_insert_drop_e == 1)
@@ -952,7 +1088,8 @@ static inline void record_drop_e(struct ppm_consumer_t *consumer, struct timespe
 
 static inline void record_drop_x(struct ppm_consumer_t *consumer, struct timespec *ts)
 {
-	if (record_event_consumer(consumer, PPME_DROP_X, NULL, -1, UF_NEVER_DROP, NULL, NULL, ts) == 0) {
+	struct event_data_t event_data = {0};
+	if (record_event_consumer(consumer, PPME_DROP_X, UF_NEVER_DROP, ts, &event_data) == 0) {
 		consumer->need_to_insert_drop_x = 1;
 	} else {
 		if (consumer->need_to_insert_drop_x == 1)
@@ -994,11 +1131,8 @@ static inline int drop_event(struct ppm_consumer_t *consumer, enum ppm_event_typ
 }
 
 static void record_event_all_consumers(enum ppm_event_type event_type,
-	struct pt_regs *regs,
-	long id,
 	enum syscall_flags drop_flags,
-	struct task_struct *sched_prev,
-	struct task_struct *sched_next)
+	struct event_data_t *event_datap)
 {
 	struct ppm_consumer_t *consumer;
 	struct timespec ts;
@@ -1007,7 +1141,7 @@ static void record_event_all_consumers(enum ppm_event_type event_type,
 
 	rcu_read_lock();
 	list_for_each_entry_rcu(consumer, &g_consumer_list, node) {
-		record_event_consumer(consumer, event_type, regs, id, drop_flags, sched_prev, sched_next, &ts);
+		record_event_consumer(consumer, event_type, drop_flags, &ts, event_datap);
 	}
 	rcu_read_unlock();
 }
@@ -1017,12 +1151,9 @@ static void record_event_all_consumers(enum ppm_event_type event_type,
  */
 static int record_event_consumer(struct ppm_consumer_t *consumer,
 	enum ppm_event_type event_type,
-	struct pt_regs *regs,
-	long id,
 	enum syscall_flags drop_flags,
-	struct task_struct *sched_prev,
-	struct task_struct *sched_next,
-	struct timespec *ts)
+	struct timespec *ts,
+	struct event_data_t *event_datap)
 {
 	int res = 0;
 	size_t event_size;
@@ -1065,13 +1196,17 @@ static int record_event_consumer(struct ppm_consumer_t *consumer,
 	}
 
 	ring_info->n_evts++;
-	if (sched_prev != NULL) {
+	if (event_datap->category == PPMC_CONTEXT_SWITCH && event_datap->event_info.context_data.sched_prev != NULL) {
 		if (event_type != PPME_SYSDIGEVENT_E) {
-			ASSERT(sched_prev != NULL);
-			ASSERT(sched_next != NULL);
-			ASSERT(regs == NULL);
+			ASSERT(event_datap->event_info.context_data.sched_prev != NULL);
+			ASSERT(event_datap->event_info.context_data.sched_next != NULL);
+			//ASSERT(regs == NULL);
 			ring_info->n_context_switches++;
 		}
+	} else if (event_datap->category == PPMC_SIGNAL) {
+		if (event_type == PPME_SIGNALDELIVER_E) {
+			ASSERT(event_datap->event_info.signal_data.info != NULL);
+		}
 	}
 
 	/*
@@ -1116,10 +1251,10 @@ static int record_event_consumer(struct ppm_consumer_t *consumer,
 	 * second argument contains a pointer to the arguments of the original
 	 * call. I guess this was done to reduce the number of syscalls...
 	 */
-	if (regs && id == __NR_socketcall) {
+	if (event_datap->category == PPMC_SYSCALL && event_datap->event_info.syscall_data.regs && event_datap->event_info.syscall_data.id == __NR_socketcall) {
 		enum ppm_event_type tet;
 
-		tet = parse_socketcall(&args, regs);
+		tet = parse_socketcall(&args, event_datap->event_info.syscall_data.regs);
 
 		if (event_type == PPME_GENERIC_E)
 			event_type = tet;
@@ -1163,10 +1298,48 @@ static int record_event_consumer(struct ppm_consumer_t *consumer,
 #endif
 		args.buffer_size = min(freespace, delta_from_end) - sizeof(struct ppm_evt_hdr); /* freespace is guaranteed to be bigger than sizeof(struct ppm_evt_hdr) */
 		args.event_type = event_type;
-		args.regs = regs;
-		args.sched_prev = sched_prev;
-		args.sched_next = sched_next;
-		args.syscall_id = id;
+
+		if (event_datap->category == PPMC_SYSCALL) {
+			args.regs = event_datap->event_info.syscall_data.regs;
+			args.syscall_id = event_datap->event_info.syscall_data.id;
+		} else {
+			args.regs = NULL;
+			args.syscall_id = -1;
+		}
+
+		if (event_datap->category == PPMC_CONTEXT_SWITCH) {
+			args.sched_prev = event_datap->event_info.context_data.sched_prev;
+			args.sched_next = event_datap->event_info.context_data.sched_next;
+		} else {
+			args.sched_prev = NULL;
+			args.sched_next = NULL;
+		}
+
+		if (event_datap->category == PPMC_SIGNAL) {
+			args.signo = event_datap->event_info.signal_data.sig;
+
+			if (args.signo == SIGKILL) {
+				args.spid = event_datap->event_info.signal_data.info->_sifields._kill._pid;
+			} else if (args.signo == SIGTERM || args.signo == SIGHUP || args.signo == SIGINT ||
+					args.signo == SIGTSTP || args.signo == SIGQUIT) {
+				if (event_datap->event_info.signal_data.info->si_code == SI_USER ||
+						event_datap->event_info.signal_data.info->si_code == SI_QUEUE ||
+						event_datap->event_info.signal_data.info->si_code <= 0) {
+					args.spid = event_datap->event_info.signal_data.info->si_pid;
+				}
+			} else if (args.signo == SIGCHLD) {
+				args.spid = event_datap->event_info.signal_data.info->_sifields._sigchld._pid;
+			} else if (args.signo >= SIGRTMIN && args.signo <= SIGRTMAX) {
+				args.spid = event_datap->event_info.signal_data.info->_sifields._rt._pid;
+			} else {
+				args.spid = (__kernel_pid_t) 0;
+			}
+		} else {
+			args.signo = 0;
+			args.spid = (__kernel_pid_t) 0;
+		}
+		args.dpid = current->pid;
+
 		args.curarg = 0;
 		args.arg_data_size = args.buffer_size - args.arg_data_offset;
 		args.nevents = ring->nevents;
@@ -1256,7 +1429,6 @@ static int record_event_consumer(struct ppm_consumer_t *consumer,
 		}
 	}
 
-#ifdef _DEBUG
 	if (ts->tv_sec > ring->last_print_time.tv_sec + 1) {
 		vpr_info("consumer:%p CPU:%d, use:%d%%, ev:%llu, dr_buf:%llu, dr_pf:%llu, pr:%llu, cs:%llu\n",
 			   consumer->consumer_id,
@@ -1270,7 +1442,6 @@ static int record_event_consumer(struct ppm_consumer_t *consumer,
 
 		ring->last_print_time = *ts;
 	}
-#endif
 
 	atomic_dec(&ring->preempt_count);
 	put_cpu();
@@ -1294,6 +1465,7 @@ TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
 
 	table_index = id - SYSCALL_TABLE_ID0;
 	if (likely(table_index >= 0 && table_index < SYSCALL_TABLE_SIZE)) {
+		struct event_data_t event_data;
 		int used = g_syscall_table[table_index].flags & UF_USED;
 		enum syscall_flags drop_flags = g_syscall_table[table_index].flags;
 		enum ppm_event_type type;
@@ -1309,10 +1481,14 @@ TRACEPOINT_PROBE(syscall_enter_probe, struct pt_regs *regs, long id)
 		type = g_syscall_table[table_index].enter_event_type;
 #endif
 
+		event_data.category = PPMC_SYSCALL;
+		event_data.event_info.syscall_data.regs = regs;
+		event_data.event_info.syscall_data.id = id;
+
 		if (used)
-			record_event_all_consumers(type, regs, id, drop_flags, NULL, NULL);
+			record_event_all_consumers(type, drop_flags, &event_data);
 		else
-			record_event_all_consumers(PPME_GENERIC_E, regs, id, UF_ALWAYS_DROP, NULL, NULL);
+			record_event_all_consumers(PPME_GENERIC_E, UF_ALWAYS_DROP, &event_data);
 	}
 }
 
@@ -1335,6 +1511,7 @@ TRACEPOINT_PROBE(syscall_exit_probe, struct pt_regs *regs, long ret)
 
 	table_index = id - SYSCALL_TABLE_ID0;
 	if (likely(table_index >= 0 && table_index < SYSCALL_TABLE_SIZE)) {
+		struct event_data_t event_data;
 		int used = g_syscall_table[table_index].flags & UF_USED;
 		enum syscall_flags drop_flags = g_syscall_table[table_index].flags;
 		enum ppm_event_type type;
@@ -1350,10 +1527,14 @@ TRACEPOINT_PROBE(syscall_exit_probe, struct pt_regs *regs, long ret)
 		type = g_syscall_table[table_index].exit_event_type;
 #endif
 
+		event_data.category = PPMC_SYSCALL;
+		event_data.event_info.syscall_data.regs = regs;
+		event_data.event_info.syscall_data.id = id;
+
 		if (used)
-			record_event_all_consumers(type, regs, id, drop_flags, NULL, NULL);
+			record_event_all_consumers(type, drop_flags, &event_data);
 		else
-			record_event_all_consumers(PPME_GENERIC_X, regs, id, UF_ALWAYS_DROP, NULL, NULL);
+			record_event_all_consumers(PPME_GENERIC_X, UF_ALWAYS_DROP, &event_data);
 	}
 }
 
@@ -1362,6 +1543,8 @@ int __access_remote_vm(struct task_struct *t, struct mm_struct *mm, unsigned lon
 
 TRACEPOINT_PROBE(syscall_procexit_probe, struct task_struct *p)
 {
+	struct event_data_t event_data;
+
 	if (unlikely(current->flags & PF_KTHREAD)) {
 		/*
 		 * We are not interested in kernel threads
@@ -1369,7 +1552,11 @@ TRACEPOINT_PROBE(syscall_procexit_probe, struct task_struct *p)
 		return;
 	}
 
-	record_event_all_consumers(PPME_PROCEXIT_1_E, NULL, -1, UF_NEVER_DROP, p, p);
+	event_data.category = PPMC_CONTEXT_SWITCH;
+	event_data.event_info.context_data.sched_prev = p;
+	event_data.event_info.context_data.sched_next = p;
+
+	record_event_all_consumers(PPME_PROCEXIT_1_E, UF_NEVER_DROP, &event_data);
 }
 
 #include <linux/ip.h>
@@ -1383,12 +1570,25 @@ TRACEPOINT_PROBE(sched_switch_probe, struct rq *rq, struct task_struct *prev, st
 TRACEPOINT_PROBE(sched_switch_probe, struct task_struct *prev, struct task_struct *next)
 #endif
 {
-	record_event_all_consumers(PPME_SCHEDSWITCH_6_E,
-		NULL,
-		-1,
-		UF_USED,
-		prev,
-		next);
+	struct event_data_t event_data;
+	event_data.category = PPMC_CONTEXT_SWITCH;
+	event_data.event_info.context_data.sched_prev = prev;
+	event_data.event_info.context_data.sched_next = next;
+
+	record_event_all_consumers(PPME_SCHEDSWITCH_6_E, UF_USED, &event_data);
+}
+#endif
+
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+TRACEPOINT_PROBE(signal_deliver_probe, int sig, struct siginfo *info, struct k_sigaction *ka)
+{
+	struct event_data_t event_data;
+	event_data.category = PPMC_SIGNAL;
+	event_data.event_info.signal_data.sig = sig;
+	event_data.event_info.signal_data.info = info;
+	event_data.event_info.signal_data.ka = ka;
+
+	record_event_all_consumers(PPME_SIGNALDELIVER_E, UF_USED | UF_ALWAYS_DROP, &event_data);
 }
 #endif
 
@@ -1483,6 +1683,10 @@ static void visit_tracepoint(struct tracepoint *tp, void *priv)
 	else if (!strcmp(tp->name, "sched_switch"))
 		tp_sched_switch = tp;
 #endif
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+	else if (!strcmp(tp->name, "signal_deliver"))
+		tp_signal_deliver = tp;
+#endif
 }
 
 static int get_tracepoint_handles(void)
@@ -1507,6 +1711,12 @@ static int get_tracepoint_handles(void)
 		return -ENOENT;
 	}
 #endif
+#ifdef CAPTURE_SIGNAL_DELIVERIES
+	if (!tp_signal_deliver) {
+		pr_err("failed to find signal_deliver tracepoint\n");
+		return -ENOENT;
+	}
+#endif
 
 	return 0;
 }
