@@ -388,6 +388,16 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #define PPME_IS_EXIT(x) (x & PPME_DIRECTION_FLAG)
 #define PPME_MAKE_ENTER(x) (x & (~1))
 
+/*
+ * Event category to classify events in generic categories
+ */
+enum ppm_capture_category {
+	PPMC_NONE = 0,
+	PPMC_SYSCALL = 1,
+	PPMC_CONTEXT_SWITCH = 2,
+	PPMC_SIGNAL = 3,
+};
+
 /** @defgroup etypes Event Types
  *  @{
  */
@@ -624,7 +634,15 @@ enum ppm_event_type {
 	PPME_CONTAINER_X = 229,
 	PPME_SYSCALL_EXECVE_16_E = 230,
 	PPME_SYSCALL_EXECVE_16_X = 231,
-	PPM_EVENT_MAX = 232
+	PPME_SIGNALDELIVER_E = 232,
+	PPME_SIGNALDELIVER_X = 233, /* This should never be called */
+	PPME_PROCINFO_E = 234,
+	PPME_PROCINFO_X = 235,	/* This should never be called */
+	PPME_SYSCALL_GETDENTS_E = 236,
+	PPME_SYSCALL_GETDENTS_X = 237,
+	PPME_SYSCALL_GETDENTS64_E = 238,
+	PPME_SYSCALL_GETDENTS64_X = 239,
+	PPM_EVENT_MAX = 240
 };
 /*@}*/
 
@@ -971,7 +989,7 @@ enum ppm_event_category {
 	EC_IO_WRITE = 33,/* General I/O write (can be file, socket, IPC...) */
 	EC_IO_OTHER = 34,/* General I/O that is neither read not write (can be file, socket, IPC...) */
 	EC_WAIT = 64,	/* General wait (can be file, socket, IPC...) */
-	EC_SCHEDULER = 128,	/* General wait (can be file, socket, IPC...) */
+	EC_SCHEDULER = 128,	/* Scheduler event (e.g. context switch) */
 	EC_INTERNAL = 256,	/* Internal event that shouldn't be shown to the user */
 };
 
@@ -1040,8 +1058,10 @@ enum ppm_param_type {
 	PT_FLAGS8 = 28, /* this is an UINT8, but will be interpreted as 8 bit flags. */
 	PT_FLAGS16 = 29, /* this is an UINT16, but will be interpreted as 16 bit flags. */
 	PT_FLAGS32 = 30, /* this is an UINT32, but will be interpreted as 32 bit flags. */
-	PT_UID = 31, /* this is an UINT32, MAX_UINT32 will be interpreted as no value */
-	PT_GID = 32 /* this is an UINT32, MAX_UINT32 will be interpreted as no value */
+	PT_UID = 31, /* this is an UINT32, MAX_UINT32 will be interpreted as no value. */
+	PT_GID = 32, /* this is an UINT32, MAX_UINT32 will be interpreted as no value. */
+	PT_DOUBLE = 33, /* this is a double precision floating point number. */
+	PT_MAX = 34 /* array size */
 };
 
 enum ppm_print_format {
@@ -1049,6 +1069,8 @@ enum ppm_print_format {
 	PF_DEC = 1,	/* decimal */
 	PF_HEX = 2,	/* hexadecima */
 	PF_10_PADDED_DEC = 3, /* decimal padded to 10 digits, useful to print the fractional part of a ns timestamp */
+	PF_ID = 4,
+	PF_DIR = 5,
 };
 
 /*!
@@ -1126,6 +1148,9 @@ struct ppm_evt_hdr {
 #define PPM_IOCTL_GET_VPID _IO(PPM_IOCTL_MAGIC, 11)
 #define PPM_IOCTL_GET_CURRENT_TID _IO(PPM_IOCTL_MAGIC, 12)
 #define PPM_IOCTL_GET_CURRENT_PID _IO(PPM_IOCTL_MAGIC, 13)
+#define PPM_IOCTL_DISABLE_SIGNAL_DELIVER _IO(PPM_IOCTL_MAGIC, 14)
+#define PPM_IOCTL_ENABLE_SIGNAL_DELIVER _IO(PPM_IOCTL_MAGIC, 15)
+#define PPM_IOCTL_GET_PROCLIST _IO(PPM_IOCTL_MAGIC, 16)
 
 /*!
   \brief System call description struct.
@@ -1164,5 +1189,19 @@ enum ppm_driver_event_id {
 	DEI_ENABLE_DROPPING = 2,
 };
 
+/*!
+  \brief Process information as returned by the PPM_IOCTL_GET_PROCLIST IOCTL.
+*/
+struct ppm_proc_info {
+	uint64_t pid;
+	uint64_t utime;
+	uint64_t stime;
+};
+
+struct ppm_proclist_info {
+	int64_t n_entries;
+	int64_t max_entries;
+	struct ppm_proc_info entries[0];
+};
 
 #endif /* EVENTS_PUBLIC_H_ */
