@@ -43,7 +43,5 @@ done
 
 echo Data saved in $DIRNAME
 
-echo Comparing
 diff -r $DIRNAME $REFERENCEDIR
-echo No change
 rm -rf $DIRNAME
