@@ -24,10 +24,11 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <dirent.h>
 #include <sys/resource.h>
 #include <sys/syscall.h>
+#include <sys/ioctl.h>
 #endif
 
 #include "scap.h"
-
+#include "../../driver/ppm_ringbuffer.h"
 #include "scap-int.h"
 
 #if defined(HAS_CAPTURE)
@@ -227,16 +228,18 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 // use prlimit to extract the RLIMIT_NOFILE for the tid. On systems where prlimit
 // is not supported, just return -1
 //
-int32_t scap_proc_fill_flimit(uint64_t tid, struct scap_threadinfo* tinfo)
+static int32_t scap_proc_fill_flimit(uint64_t tid, struct scap_threadinfo* tinfo)
 #ifdef SYS_prlimit64
 {
 	struct rlimit rl;
 
+#ifdef __NR_prlimit64
 	if(syscall(SYS_prlimit64, tid, RLIMIT_NOFILE, NULL, &rl) == 0)
 	{
 		tinfo->fdlimit = rl.rlim_cur;
 		return SCAP_SUCCESS;
 	}
+#endif
 
 	tinfo->fdlimit = -1;
 	return SCAP_SUCCESS;
@@ -248,21 +251,178 @@ int32_t scap_proc_fill_flimit(uint64_t tid, struct scap_threadinfo* tinfo)
 }
 #endif
 
+int32_t scap_proc_fill_cgroups(struct scap_threadinfo* tinfo, const char* procdirname)
+{
+	char filename[SCAP_MAX_PATH_SIZE];
+	char line[SCAP_MAX_CGROUPS_SIZE];
+
+	tinfo->cgroups_len = 0;
+	snprintf(filename, sizeof(filename), "%scgroup", procdirname);
+
+    if(access(filename, R_OK) == -1)
+	{
+		return SCAP_SUCCESS;
+	}
+
+	FILE* f = fopen(filename, "r");
+	if(f == NULL)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+	while(fgets(line, sizeof(line), f) != NULL)
+	{
+		char* token;
+		char* subsys_list;
+		char* cgroup;
+
+		// id
+		token = strtok(line, ":");
+		if(token == NULL)
+		{
+			ASSERT(false);
+			fclose(f);
+			return SCAP_FAILURE;
+		}
+
+		// subsys
+		subsys_list = strtok(NULL, ":");
+		if(subsys_list == NULL)
+		{
+			ASSERT(false);
+			fclose(f);
+			return SCAP_FAILURE;
+		}
+
+		// transient cgroup
+		if(strncmp(subsys_list, "name=", sizeof("name=") - 1) == 0)
+		{
+			continue;
+		}
+
+		// cgroup
+		cgroup = strtok(NULL, ":");
+		if(cgroup == NULL)
+		{
+			ASSERT(false);
+			fclose(f);
+			return SCAP_FAILURE;
+		}
+
+		// remove the \n
+		cgroup[strlen(cgroup) - 1] = 0;
+
+		while((token = strtok(subsys_list, ",")) != NULL)
+		{
+			subsys_list = NULL;
+			if(strlen(cgroup) + 1 + strlen(token) + 1 > SCAP_MAX_CGROUPS_SIZE - tinfo->cgroups_len)
+			{
+				ASSERT(false);
+				fclose(f);
+				return SCAP_SUCCESS;
+			}
+
+			snprintf(tinfo->cgroups + tinfo->cgroups_len, SCAP_MAX_CGROUPS_SIZE - tinfo->cgroups_len, "%s=%s", token, cgroup);
+			tinfo->cgroups_len += strlen(cgroup) + 1 + strlen(token) + 1;			
+		}
+	}
+
+	fclose(f);
+	return SCAP_SUCCESS;
+}
+
+static int32_t scap_get_vtid(scap_t* handle, int64_t tid, int64_t *vtid)
+{
+	if(handle->m_file)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+#if !defined(HAS_CAPTURE)
+	ASSERT(false)
+	return SCAP_FAILURE;
+#else
+
+	*vtid = ioctl(handle->m_devs[0].m_fd, PPM_IOCTL_GET_VTID, tid);
+
+	if(*vtid == -1)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+	return SCAP_SUCCESS;
+#endif
+}
+
+static int32_t scap_get_vpid(scap_t* handle, int64_t tid, int64_t *vpid)
+{
+	if(handle->m_file)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+#if !defined(HAS_CAPTURE)
+	ASSERT(false)
+	return SCAP_FAILURE;
+#else
+
+	*vpid = ioctl(handle->m_devs[0].m_fd, PPM_IOCTL_GET_VPID, tid);
+
+	if(*vpid == -1)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+	return SCAP_SUCCESS;
+#endif
+}
+
+int32_t scap_getpid_global(scap_t* handle, int64_t* pid)
+{
+	if(handle->m_file)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+#if !defined(HAS_CAPTURE)
+	ASSERT(false)
+	return SCAP_FAILURE;
+#else
+
+	*pid = ioctl(handle->m_devs[0].m_fd, PPM_IOCTL_GET_CURRENT_PID);
+	if(*pid == -1)
+	{
+		ASSERT(false);
+		return SCAP_FAILURE;
+	}
+
+	return SCAP_SUCCESS;
+#endif	
+}
+
 //
 // Add a process to the list by parsing its entry under /proc
 //
-int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int tid_to_scan, char* procdirname, scap_fdinfo* sockets, scap_threadinfo** procinfo, char *error)
+static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int tid_to_scan, char* procdirname, struct scap_ns_socket_list** sockets_by_ns, scap_threadinfo** procinfo, char *error)
 {
 	char dir_name[256];
 	char target_name[256];
 	int target_res;
 	char filename[252];
-	char line[SCAP_MAX_PATH_SIZE];
+	char line[SCAP_MAX_ENV_SIZE];
 	struct scap_threadinfo* tinfo;
 	int32_t uth_status = SCAP_SUCCESS;
 	FILE* f;
 	size_t filesize;
 	size_t exe_len;
+	bool free_tinfo = false;
+	int32_t res = SCAP_SUCCESS;
 
 	snprintf(dir_name, sizeof(dir_name), "%s/%u/", procdirname, tid);
 	snprintf(filename, sizeof(filename), "%sexe", dir_name);
@@ -287,6 +447,8 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 			return SCAP_SUCCESS;
 		}
 
+		ASSERT(sizeof(line) >= SCAP_MAX_PATH_SIZE);
+
 		if(fgets(line, SCAP_MAX_PATH_SIZE, f) == NULL)
 		{
 			fclose(f);
@@ -298,8 +460,6 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 		}
 	}
 
-	target_name[target_res] = 0;
-
 	//
 	// This is a real user level process. Allocate the procinfo structure.
 	//
@@ -339,8 +499,6 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 		tinfo->flags = PPM_CL_CLONE_THREAD | PPM_CL_CLONE_FILES;
 	}
 
-	snprintf(tinfo->exe, SCAP_MAX_PATH_SIZE, "%s", target_name);
-
 	//
 	// Gather the command name
 	//
@@ -355,6 +513,8 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 	}
 	else
 	{
+		ASSERT(sizeof(line) >= SCAP_MAX_PATH_SIZE);
+
 		if(fgets(line, SCAP_MAX_PATH_SIZE, f) == NULL)
 		{
 			snprintf(error, SCAP_LASTERR_SIZE, "can't read from %s", filename);
@@ -382,24 +542,32 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 	}
 	else
 	{
-		filesize = fread(line, 1, sizeof(line), f);
-		line[filesize - 1] = 0;
+		ASSERT(sizeof(line) >= SCAP_MAX_ARGS_SIZE);
 
-		exe_len = strlen(line);
-		if(exe_len < filesize)
+		filesize = fread(line, 1, SCAP_MAX_ARGS_SIZE - 1, f);
+		if(filesize > 0)
 		{
-			++exe_len;
-		}
+			line[filesize] = 0;
 
-		tinfo->args_len = filesize - exe_len;
-		if(tinfo->args_len > SCAP_MAX_PATH_SIZE)
+			exe_len = strlen(line);
+			if(exe_len < filesize)
+			{
+				++exe_len;
+			}
+
+			snprintf(tinfo->exe, SCAP_MAX_PATH_SIZE, "%s", line);
+
+			tinfo->args_len = filesize - exe_len;
+
+			memcpy(tinfo->args, line + exe_len, tinfo->args_len);
+			tinfo->args[SCAP_MAX_ARGS_SIZE - 1] = 0;
+		}
+		else
 		{
-			tinfo->args_len = SCAP_MAX_PATH_SIZE;
+			tinfo->args[0] = 0;
+			tinfo->exe[0] = 0;
 		}
 
-		memcpy(tinfo->args, line + exe_len, tinfo->args_len);
-		tinfo->args[SCAP_MAX_PATH_SIZE - 1] = 0;
-
 		fclose(f);
 	}
 
@@ -417,17 +585,23 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 	}
 	else
 	{
-		filesize = fread(line, 1, sizeof(line), f);
-		line[filesize - 1] = 0;
+		ASSERT(sizeof(line) >= SCAP_MAX_ENV_SIZE);
 
-		tinfo->env_len = filesize;
-		if(tinfo->env_len > SCAP_MAX_ENV_SIZE)
+		filesize = fread(line, 1, SCAP_MAX_ENV_SIZE, f);
+
+		if(filesize > 0)
 		{
-			tinfo->env_len = SCAP_MAX_ENV_SIZE;
-		}
+			line[filesize - 1] = 0;
 
-		memcpy(tinfo->env, line, tinfo->env_len);
-		tinfo->env[SCAP_MAX_ENV_SIZE - 1] = 0;
+			tinfo->env_len = filesize;
+
+			memcpy(tinfo->env, line, tinfo->env_len);
+			tinfo->env[SCAP_MAX_ENV_SIZE - 1] = 0;
+		}
+		else
+		{
+			tinfo->env[0] = 0;
+		}
 
 		fclose(f);
 	}
@@ -462,20 +636,45 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 		return SCAP_FAILURE;
 	}
 
+	if(scap_proc_fill_cgroups(tinfo, dir_name) == SCAP_FAILURE)
+	{
+		snprintf(error, SCAP_LASTERR_SIZE, "can't fill cgroups for %" PRIu64, tinfo->tid);
+		free(tinfo);
+		return SCAP_FAILURE;	
+	}
+
+	if(scap_get_vtid(handle, tinfo->tid, &tinfo->vtid) == SCAP_FAILURE)
+	{
+		tinfo->vtid = -1;
+	}
+
+	if(scap_get_vpid(handle, tinfo->tid, &tinfo->vpid) == SCAP_FAILURE)
+	{
+		tinfo->vpid = -1;
+	}
+
 	//
-	// if tid_to_scan is set we assume is a runtime lookup so no
+	// if tid_to_scan is set we assume this is a runtime lookup so no
 	// need to use the table
 	//
 	if(tid_to_scan == -1)
 	{
 		//
-		// Done. Add the entry to the process table
+		// Done. Add the entry to the process table, or fire the notification callback
 		//
-		HASH_ADD_INT64(handle->m_proclist, tid, tinfo);
-		if(uth_status != SCAP_SUCCESS)
+		if(handle->m_proc_callback == NULL)
 		{
-			snprintf(error, SCAP_LASTERR_SIZE, "process table allocation error (2)");
-			return SCAP_FAILURE;
+			HASH_ADD_INT64(handle->m_proclist, tid, tinfo);
+			if(uth_status != SCAP_SUCCESS)
+			{
+				snprintf(error, SCAP_LASTERR_SIZE, "process table allocation error (2)");
+				return SCAP_FAILURE;
+			}
+		}
+		else
+		{
+			handle->m_proc_callback(handle->m_proc_callback_context, tinfo->tid, tinfo, NULL, handle);
+			free_tinfo = true;
 		}
 	}
 	else
@@ -486,12 +685,17 @@ int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parenttid, int
 	//
 	// Only add fds for processes, not threads
 	//
-	if(-1 == parenttid)
+	if(parenttid == -1)
 	{
-		return scap_fd_scan_fd_dir(handle, dir_name, tinfo, sockets, error);
+		res = scap_fd_scan_fd_dir(handle, dir_name, tinfo, sockets_by_ns, error);
 	}
 
-	return SCAP_SUCCESS;
+	if(free_tinfo)
+	{
+		free(tinfo);
+	}
+
+	return res;
 }
 
 //
@@ -506,7 +710,7 @@ int32_t scap_proc_scan_proc_dir(scap_t* handle, char* procdirname, int parenttid
 	int32_t res = SCAP_SUCCESS;
 	char childdir[SCAP_MAX_PATH_SIZE];
 
-	scap_fdinfo* sockets = NULL;
+	struct scap_ns_socket_list* sockets_by_ns = NULL;
 
 	tid = 0;
 	dir_p = opendir(procdirname);
@@ -519,13 +723,9 @@ int32_t scap_proc_scan_proc_dir(scap_t* handle, char* procdirname, int parenttid
 
 	if(-1 == parenttid)
 	{
-		if(scan_sockets)
+		if(!scan_sockets)
 		{
-			if(SCAP_FAILURE == scap_fd_read_sockets(handle, &sockets))
-			{
-				closedir(dir_p);
-				return SCAP_FAILURE;
-			}
+			sockets_by_ns = (void*)-1;
 		}
 	}
 
@@ -575,7 +775,7 @@ int32_t scap_proc_scan_proc_dir(scap_t* handle, char* procdirname, int parenttid
 			//
 			// We have a process that needs to be explored
 			//
-			res = scap_proc_add_from_proc(handle, tid, parenttid, tid_to_scan, procdirname, sockets, procinfo, error);
+			res = scap_proc_add_from_proc(handle, tid, parenttid, tid_to_scan, procdirname, &sockets_by_ns, procinfo, error);
 			if(res != SCAP_SUCCESS)
 			{
 				snprintf(error, SCAP_LASTERR_SIZE, "cannot add procs tid = %"PRIu64", parenttid = %"PRIi32", dirname = %s", tid, parenttid, procdirname);
@@ -614,7 +814,10 @@ int32_t scap_proc_scan_proc_dir(scap_t* handle, char* procdirname, int parenttid
 	}
 
 	closedir(dir_p);
-	scap_fd_free_table(handle, &sockets);
+	if(sockets_by_ns != NULL && sockets_by_ns != (void*)-1)
+	{
+		scap_fd_free_ns_sockets_list(handle, &sockets_by_ns);
+	}
 	return res;
 }
 
@@ -670,8 +873,9 @@ struct scap_threadinfo* scap_proc_get(scap_t* handle, int64_t tid, bool scan_soc
 	}
 
 	struct scap_threadinfo* tinfo = NULL;
-
-	if(scap_proc_scan_proc_dir(handle, "/proc", -1, tid, &tinfo, handle->m_lasterr, scan_sockets) != SCAP_SUCCESS)
+	char filename[SCAP_MAX_PATH_SIZE];
+	snprintf(filename, sizeof(filename), "%s/proc", scap_get_host_root());
+	if(scap_proc_scan_proc_dir(handle, filename, -1, tid, &tinfo, handle->m_lasterr, scan_sockets) != SCAP_SUCCESS)
 	{
 		return NULL;
 	}
@@ -680,6 +884,45 @@ struct scap_threadinfo* scap_proc_get(scap_t* handle, int64_t tid, bool scan_soc
 #endif // HAS_CAPTURE
 }
 
+bool scap_is_thread_alive(scap_t* handle, int64_t pid, int64_t tid, const char* comm)
+{
+#if !defined(HAS_CAPTURE)
+	return false;
+#else
+	char charbuf[SCAP_MAX_PATH_SIZE];
+	FILE* f;
+
+	//
+	// No /proc parsing for offline captures
+	//
+	if(handle->m_file)
+	{
+		ASSERT(false);
+		return false;
+	}
+
+	snprintf(charbuf, sizeof(charbuf), "%s/proc/%" PRId64 "/task/%" PRId64 "/comm", scap_get_host_root(), pid, tid);
+
+	f = fopen(charbuf, "r");
+
+	if(f != NULL)
+	{
+		if(fgets(charbuf, sizeof(charbuf), f) != NULL)
+		{
+			if(strncmp(charbuf, comm, strlen(comm)) == 0)
+			{
+				fclose(f);
+				return true;
+			}
+		}
+
+		fclose(f);
+	}
+
+	return false;
+#endif // HAS_CAPTURE
+}
+
 void scap_proc_free(scap_t* handle, struct scap_threadinfo* proc)
 {
 	scap_fd_free_proc_fd_table(handle, proc);
