@@ -20,9 +20,14 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <limits.h>
 #include <stdlib.h>
+#include <sys/time.h>
+#include <execinfo.h>
+#include <unistd.h>
+#include <sys/time.h>
 #endif
 #include <algorithm> 
 #include <functional> 
+#include <errno.h>
 
 #include "sinsp.h"
 #include "sinsp_int.h"
@@ -30,6 +35,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "sinsp_signal.h"
 #include "filter.h"
 #include "filterchecks.h"
+#include "chisel.h"
 #include "protodecoder.h"
 
 #ifndef PATH_MAX
@@ -727,6 +733,38 @@ void sinsp_utils::get_filtercheck_fields_info(OUT vector<const filter_check_info
 }
 #endif
 
+uint64_t sinsp_utils::get_current_time_ns()
+{
+    struct timeval tv;
+    gettimeofday(&tv, NULL);
+
+    return tv.tv_sec * (uint64_t) 1000000000 + tv.tv_usec * 1000;
+}
+
+#ifndef _WIN32
+void sinsp_utils::bt(void)
+{
+	static const char start[] = "BACKTRACE ------------";
+	static const char end[] = "----------------------";
+
+	void *bt[1024];
+	int bt_size;
+	char **bt_syms;
+	int i;
+
+	bt_size = backtrace(bt, 1024);
+	bt_syms = backtrace_symbols(bt, bt_size);
+	g_logger.format("%s", start);
+	for (i = 1; i < bt_size; i++) 
+	{
+		g_logger.format("%s", bt_syms[i]);
+	}
+	g_logger.format("%s", end);
+
+	free(bt_syms);
+}
+#endif // _WIN32
+
 ///////////////////////////////////////////////////////////////////////////////
 // gettimeofday() windows implementation
 ///////////////////////////////////////////////////////////////////////////////
