@@ -76,6 +76,15 @@ public:
 	// Get the list of filtercheck fields
 	//
 	static void get_filtercheck_fields_info(vector<const filter_check_info*>* list);
+
+	static uint64_t get_current_time_ns();
+
+#ifndef _WIN32
+	//
+	// Print the call stack
+	//
+	static void bt(void);
+#endif // _WIN32
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -124,6 +133,8 @@ string ipv6serveraddr_to_string(ipv6serverinfo* addr);
 // String helpers
 ///////////////////////////////////////////////////////////////////////////////
 vector<string> sinsp_split(const string &s, char delim);
+template<typename It>
+string sinsp_join(It begin, It end, char delim);
 string& ltrim(string &s);
 string& rtrim(string &s);
 string& trim(string &s);
